if rebasics then return end

-- To use colorized hud, this needs a table with skincolor color translations. There is a convinient one in libsg in Sunflower's Garden lua,
-- but if you want it to be available without this pk3, you can port it from hardcode
local translations = SG_Translations

rawset(_G, "rebasics", true)

freeslot("sfx_rbtrk0", "sfx_rbtrk1", "sfx_rbtrk2", "sfx_rbtrk3")

local themes = {
	default = {
		trick = { sfx_rbtrk0, },
		release = { sfx_rbtrk1, sfx_rbtrk2, sfx_rbtrk3, },
		releasex = nil, -- Extra sound that is played along with each release one (yes its mostly exists for acrobasics theme)
	},
}

local curtheme = "default"

local starttime = 6*TICRATE + (3*TICRATE/4)

-- Boost timer isn't actually capped in any ways, those are just values that are used to "scale" boost, timer consumption rate, etc
local BOOSTTIMER_MIN = TICRATE
local BOOSTTIMER_MAX = 10*TICRATE

local BOOSTTIMER_ADD = 3*TICRATE*FRACUNIT/2

local GLOATGRADE = 4 -- CHARGE!

local GAUGE_WIDTH = 50
local GAUGE_HEIGHT = 4
local GAUGE_SPACE = 1

local SMOLGAUGE_WIDTH = 20
local SMOLGAUGE_HEIGHT = 2
local SMOLGAUGE_SPACE = 0

-- Those levels are somewhat arbitrary, but still kinda help represent boost strength:
-- green level is normal, thats what you get after single trick
-- blue level gives increased speed and increased gauge usage, usually you get that with 3 or more tricks
-- purple trick is hard to get (because it gets "eaten" very quickly) and usually makes it really hard to control chars even with high speed stat (so with weaker boost)
local GAUGE_LEVELS = {
    -- Normal trick gauge color, gets replaced with yellow and red if timer is too low, like in acrobasics
    { len = 2*TICRATE*FRACUNIT, colors = { 160, }, },

    -- Fancy animated blue
    { len = 2*TICRATE*FRACUNIT, colors = { 229, 228, 227, 226, 225, 0, 226, 227, 228, }, },

    -- Fancy animated purple
    { len = 4*TICRATE*FRACUNIT, colors = { 196, 195, 194, 193, 192, 0, 0, 192, 193, 194, 195 }, }
}

local cv_enabled = CV_RegisterVar {
    name = "rebasics",
    defaultvalue = "Off",
    possiblevalue = CV_OnOff,
    flags = CV_NETVAR,
}

local cv_trickcooldown = CV_RegisterVar {
    name = "rb_trickcooldown",
    defaultvalue = 10,
    possiblevalue = CV_Unsigned,
    flags = CV_NETVAR,
}

-- Yep thats taken from sg acrobasics (removed config stuff for now, will probably re-add later)
local cv_trickhudcolor = CV_RegisterVar {
    name = "rb_trickhudcolor",
    defaultvalue = "Off",
    possiblevalue = CV_OnOff,

    --config_menu = "Acrobasics",
    --displayname = "Trick HUD Color",
	--description = "Colorize trick HUD background. Inverted if colorizedhud is enabled."
}

-- grrr
local function K_FlipFromObject(mo, master)
	mo.eflags = (mo.eflags & ~MFE_VERTICALFLIP)|(master.eflags & MFE_VERTICALFLIP)
	mo.flags2 = (mo.flags2 & ~MF2_OBJECTFLIP)|(master.flags2 & MF2_OBJECTFLIP)

	if mo.eflags & MFE_VERTICALFLIP then
		mo.z = mo.z + master.height - FixedMul(master.scale, mo.height)
    end
end

-- GRRR
local function P_IsPlayerLocal(p)
	if p == consoleplayer return true end

	for i = 1, 3 do
		if p == displayplayers[i] then return true end
	end
	
	return false
end

local function rescale(val, oldmin, oldmax, newmin, newmax)
    return newmin + FixedMul(FixedDiv(val-oldmin, oldmax-oldmin), newmax-newmin)
end

local function clamp(minval, val, maxval)
    return min(max(minval, val), maxval)
end

local function trickSound(combo)
	return themes[curtheme].trick[min(#themes[curtheme].trick, combo+1)]
end

local function trickReleaseSound(combo)
	return themes[curtheme].release[min(#themes[curtheme].release, combo)], themes[curtheme].releasex
end

local function trickCancelSound()
	return themes[curtheme].calcel or sfx_none
end

local function initTrickData(rb)
    rb.trickbuffer = false
	rb.trickmiss = false
    rb.trickcombo = 0

    rb.boosttimer = 0
    rb.targetspeedboost = 0
    rb.speedboost = 0
    rb.accelboost = 0

    -- Fixes trick spam from stuff like glider
    rb.lasttrick = 0

    -- ...But that fix becomes problem if there are 2 legal tricks very close to each other.
    -- Lets fix that by remembering if player wanted to trick and after cooldown, if we're still in air, we can trick
    rb.wantedtrick = false

    -- No-physics slopes can give tricks sometimes in acrobasics, but its inconsistent.
    -- To make it consistent, we will use last slope player been standing on and check: if player was moving upward on that slope, we allow them to trick
    rb.lastslope = nil

    -- Speeeeeen
    rb.spin = 0
end

local function trickData(pmo, nocreate)
    local data = pmo.rebasics

    if not data and not nocreate then
        pmo.rebasics = {}
        data = pmo.rebasics

        initTrickData(data)
    end

    return data
end

local function spawnTrickVFX(player)
    -- Yep thats just fast lines for now
    -- I'd probably need to make something original sometime

    -- Game will already spawn those, so would be better to spawn less of them
	if ((player.kartstuff[k_sneakertimer] or player.kartstuff[k_driftboost] or player.kartstuff[k_startboost]) and (leveltime % 2 == 0))
        or player.speed == 0 then -- No need to spawn on 0 speed
        return
    end

    local fast = P_SpawnMobj(player.mo.x + (RB_RandomRange(-36,36) * player.mo.scale),
        player.mo.y + (RB_RandomRange(-36,36) * player.mo.scale),
        player.mo.z + (player.mo.height/2) + (RB_RandomRange(-20,20) * player.mo.scale),
        MT_FASTLINE)
    fast.angle = R_PointToAngle2(0, 0, player.mo.momx, player.mo.momy)
    fast.momx = 3*player.mo.momx/4
    fast.momy = 3*player.mo.momy/4
    fast.momz = 3*player.mo.momz/4
    fast.target = player.mo
    fast.colorized = true
    fast.color = player.mo.color

    -- Scale up depending on how much boost we have
    local tics = trickData(player.mo).boosttimer>>FRACBITS
    fast.destscale = rescale(clamp(BOOSTTIMER_MIN, tics, BOOSTTIMER_MAX), BOOSTTIMER_MIN, BOOSTTIMER_MAX, fast.destscale, fast.destscale*3) -- They can get comically large
    P_SetScale(fast, max(fast.scale, 2*fast.destscale/3))

    K_MatchGenericExtraFlags(fast, player.mo)
end

local function doTrick(pmo)
    local rb = trickData(pmo)

    if rb.trickbuffer then return end

    rb.trickbuffer = true
	rb.wantedtrick = false

    rb.spin = 8*(1 + min(rb.trickcombo, 3))

	local volume = P_IsPlayerLocal(pmo.player) and 255 or 127

	S_StartSoundAtVolume(pmo, trickSound(rb.trickcombo), volume)
end

local function cancelTrick(pmo)
	local rb = trickData(pmo)
	
	if not rb.trickbuffer then return end
	
	rb.trickbuffer = false
	rb.trickmiss = true
	rb.spin = 0
	
	if pmo.player then
		S_StartSound(pmo, trickCancelSound(), pmo.player)
	end
end

local function trickSpin(pmo)
    local p = pmo.player

    if not p then return end

    local rb = trickData(pmo)

    if rb.spin == 0 then return end

    p.frameangle = pmo.angle - rb.spin*ANGLE_45
    rb.spin = $ - 1
end


-- Ticks timer and sets target speedboost value. The more time you have, the more tics is taken and higher speedboost value is
local function consumeBoostTimer(p, rb)
    local kartspeed = clamp(1, p.kartspeed, 9) -- more speed -> less timer consumption
    local kartaccel = 10 - kartspeed -- more accel -> more speed boost
    local kartweight = clamp(1, p.kartweight, 9) -- more weight -> (slightly) more timer consumption
    local tics = rb.boosttimer>>FRACBITS

    -- Boost part
    local accel_factor = rescale(kartaccel, 1, 9, FRACUNIT, 9*FRACUNIT/4)
    rb.targetspeedboost = FixedMul(rescale(max(TICRATE, tics), BOOSTTIMER_MIN, BOOSTTIMER_MAX, FRACUNIT/4, 2*FRACUNIT), accel_factor)

    rb.accelboost = rescale(clamp(BOOSTTIMER_MIN, tics, BOOSTTIMER_MAX), BOOSTTIMER_MIN, BOOSTTIMER_MAX, 4*FRACUNIT, 12*FRACUNIT)

    -- Timer part
    local air_factor = P_IsObjectOnGround(p.mo) and FRACUNIT or FRACUNIT/8
    local amount_factor = rescale(max(BOOSTTIMER_MIN, tics), BOOSTTIMER_MIN, BOOSTTIMER_MAX, FRACUNIT, 6*FRACUNIT)
    local weight_factor = rescale(kartweight, 1, 9, FRACUNIT, 13*FRACUNIT/10)
    local speed_factor = rescale(kartspeed, 1, 9, 9*FRACUNIT/10, FRACUNIT/2)

    local consume = FixedMul(FixedMul(FixedMul(weight_factor, amount_factor), speed_factor), air_factor)
    rb.boosttimer = max(0, $-consume)
end

local function releaseTrick(pmo)
    local rb = trickData(pmo)

    if not rb.trickbuffer then return end

    rb.trickbuffer = false

    rb.lasttrick = leveltime

    rb.boosttimer = $+BOOSTTIMER_ADD

    rb.trickcombo = $ + 1

    local overlay = P_SpawnMobj(pmo.x, pmo.y, pmo.z, MT_BOOSTFLAME)
    overlay.target = pmo
    overlay.destscale = pmo.scale
    P_SetScale(overlay, overlay.destscale)
    K_FlipFromObject(overlay, pmo)

	local sfx, extrasfx = trickReleaseSound(rb.trickcombo)

    S_StartSound(pmo, sfx, pmo.player)
	if extrasfx then S_StartSound(pmo, extrasfx, pmo.player) end

    if rb.trickcombo >= GLOATGRADE then
        K_PlayPowerGloatSound(pmo)
    elseif P_RandomChance(rb.trickcombo*FRACUNIT/GLOATGRADE) then
        K_PlayBoostTaunt(pmo)
    end
end

local function trickBoost(pmo)
    local rb = trickData(pmo)
    local p = pmo.player

    if rb.speedboost == 0 and rb.targetspeedboost == 0 then return end

    if rb.speedboost < rb.targetspeedboost then
        rb.speedboost = rb.targetspeedboost
    else
        rb.speedboost = max($ - FRACUNIT/TICRATE, rb.targetspeedboost)
    end

    p.kartstuff[k_speedboost] = max($, rb.speedboost)

    if rb.accelboost ~= 0 then
        p.kartstuff[k_accelboost] = max($, rb.accelboost)
    end
end

local function isGoingUp(pmo, rb)
    local flip = pmo.eflags & MFE_VERTICALFLIP

    if (pmo.momz > 0 and not flip) or (pmo.momz < 0 and flip) then
        return true
    end

    if rb.lastslope == nil then return false end

    local slope = rb.lastslope

    local movedir = R_PointToAngle2(0, 0, pmo.momx, pmo.momy)
    local slopedir = slope.xydirection

    if (slope.zangle < 0 and flip) or (slope.zangle > 0 and not flip) then
        slopedir = slopedir + ANGLE_180
    end

    local diff = abs(movedir - slopedir)

    if diff < ANGLE_90 then
        return false
    end

    return true
end

local function canTrick(pmo)
    local p = pmo.player

    -- Can non-player mobjs do tricks? Maybe?
    if p then
        local ks = p.kartstuff

        if P_PlayerInPain(p) or max(ks[k_spinouttimer], max(ks[k_wipeoutslow], ks[k_squishedtimer])) > 0 then
            return false
        end
    end

    local rb = trickData(pmo)

    local goingup = isGoingUp(pmo, rb)

    if not goingup then return false end

    return true
end

local function trickThink(p)
    if p.spectator then return end
    if p.playerstate ~= PST_LIVE or p.deadtimer > 0 or p.kartstuff[k_respawn] > 0 then return end

    local pmo = p.mo
	local ks = p.kartstuff

    local rb = trickData(pmo)

    if not P_IsObjectOnGround(pmo) then
        local trickbutton = p.cmd.buttons & (p.rb_trickbutton or BT_BRAKE)

        if not rb.trickbuffer and trickbutton and canTrick(pmo) then
            rb.wantedtrick = true
        end

        if rb.wantedtrick
            and ((leveltime - rb.lasttrick >= cv_trickcooldown.value) -- Trick spam prevention
                or ks[k_pogospring]) then                    -- ...except for rainbow road
            doTrick(pmo)
        end

        trickSpin(pmo)

        if rb.trickbuffer then
            K_SpawnSparkleTrail(pmo)
        end
    else
        if rb.trickbuffer then
            releaseTrick(pmo)
        end

        -- Don't speen after you landed
        rb.spin = 0

        -- Wanted to trick but cooldown didn't let to? Too bad!
        rb.wantedtrick = false
    end

    if rb.boosttimer > 0 then
        consumeBoostTimer(p, rb)
        spawnTrickVFX(p)
    else
        rb.trickcombo = 0
        rb.targetspeedboost = 0
        rb.accelboost = 0
    end
	
	if not (P_PlayerInPain(p) or max(ks[k_spinouttimer], max(ks[k_wipeoutslow], ks[k_squishedtimer])) > 0) then
		rb.trickmiss = false
	end

    trickBoost(pmo)
    rb.lastslope = pmo.standingslope
end

addHook("PlayerCmd", function(p, cmd)
	if not p.mo then return end

	if not p.rb_holdaccel then return end

	local rb = trickData(p.mo, true)
	
	if not rb then return end
	
	if rb.boosttimer > 0 then
		cmd.buttons = $ | BT_ACCELERATE
		cmd.forwardmove = 50 -- Why does hardcode do 50<<FRACBITS>>16 is beyond me
	end
end)

addHook("ThinkFrame", function()
    if cv_enabled.value == 0 then return end
    if leveltime < starttime then return end

    for p in players.iterate do
        trickThink(p)
    end
end)

-- I still can't decide if non player mobjs are allowed to trick or not :AAAAAAAA:
local function calcelPlayerTrick(p)
	-- I think it is possible for p.mo to be nil here?
	if p.mo then
		cancelTrick(p.mo)
	end
end

addHook("PlayerSpin", calcelPlayerTrick)
addHook("PlayerSquish", calcelPlayerTrick)
addHook("PlayerExplode", calcelPlayerTrick)

-------------------------------------------------------------------------------------
-- HUD code
-------------------------------------------------------------------------------------

local function useColorHud(v)
    if v.useColorHud then return v.useColorHud() ~= (cv_trickhudcolor.value == 1) end
    return cv_trickhudcolor.value
end

local function getHudColor(v, p)
    if v.getHudColor then return v.getHudColor() end

    return p.skincolor
end

local function drawGaugeBackground(v, x, y, width, height, flags)
    local gaugebgcolor = 24

    if useColorHud(v) and translations then
        gaugebgcolor = translations[getHudColor(v, p)][8]
    end

	v.drawFill(x, y, width, height, gaugebgcolor|flags)
end

local function drawGauge(v, x, y, width, height, space, fill, gaugecolor, flags)
    local fill_width = FixedMul((width - space*2)*FRACUNIT, fill)/FRACUNIT
    local fill_height = GAUGE_HEIGHT - space*2

	if v.interpolate then v.interpolate(true) end
	v.drawFill(x + space, y + space, fill_width, fill_height, gaugecolor|flags)
	if v.interpolate then v.interpolate(false) end
end

-- Returns level number and fill value for drawGauge
local function getGaugeLevel(rb)
    local value = rb.boosttimer

    for i, level in ipairs(GAUGE_LEVELS) do
        if value < level.len then
            return i, FixedDiv(value, level.len)
        end

        value = value - level.len
    end

    -- Too much charge!
    return #GAUGE_LEVELS, FRACUNIT
end

local function getGaugeColor(level)
    local colors = GAUGE_LEVELS[level].colors

    return colors[(leveltime % #colors) + 1]
end

local combotext = {
    [0] = "MISS...",
    "OK...",
    "NICE!",
    "RADICAL!",
    "WILD!",
}

-- For 3p/4p splitscreen
local combocolor = {
    [0] = '\x86',
    '\x83',
    '\x84',
    '\x81',
    '\x89',
}

-- Trick font stuff. Its basically just v.drawKartString font but with '.' and '!' support :blobcatgoogly:
local font

local function cacheTrickFont(v)
    font = { [33] = v.cachePatch("TSFNTEXC"), [46] = v.cachePatch("TSFNTDOT") }
    for i = string.byte('A'), string.byte('Z') do
        font[i] = v.cachePatch(string.format("MKFNT%03d", i))
    end
end

local function trickStringWidth(v, s)
    local w = 0
    local space = 12

    for i = 1, #s do
        local c = string.byte(s, i)

        if c & 0x80 then continue end

        w = w + space
    end

    return w
end

local function drawTrickString(v, x, y, num, flags)
    local s = combotext[num]

    local anim = leveltime

    local centerx = x
    x = x - trickStringWidth(v, s)/2

    -- Don't interpolate miss animation, it looks funnier that way
    if num ~= 0 and v.interpolate then v.interpolate(num) end

    for i = 1, #s do
        local c = string.byte(s, i)

        local offx = 0
        local offy = 0

        -- Heavely inspired by GenericHeroGuy's acrobasics animation rework
        if num == 0 then
            -- Miss...
            local offsetsx = { 2, 0, 3, -2, 1, 2, -3, 1, }
            local offsetsy = { -2, 3, 2, 1, 2, 0, -2, 3 }
            offx = offsetsx[anim % #offsetsx + 1]*FRACUNIT
            offy = offsetsy[anim % #offsetsy + 1]*FRACUNIT
        elseif num == 1 then
            -- Ok...
            if (anim/3 - i) % #s == 0 then
                offy = -3*FRACUNIT/2
            end
        elseif num == 2 then
            -- Nice!
            local sign = ((i % 2) == 0) and 1 or -1
            local offsets = { -2, -4, -2, -1, 0, 1, 2, 4, 2, 1, 0, -1 }
            offy = sign*offsets[(anim/2) % #offsets + 1]*FRACUNIT
        elseif num == 3 then
            -- Radical!
            local offsetsx = { -2, -1, 0, 1, 2, 1, 0, -1, }
            local offsetsy = { 0, 1, 2, 1, 0, -1, -2, -1, }
            offx = offsetsx[(anim+i) % #offsetsx + 1]*FRACUNIT
            offy = offsetsy[(anim+i) % #offsetsy + 1]*FRACUNIT
        else
            -- Wild!
            local j = (i - #s) + #s/2

            local sign = 0

            if j < 0 then
                sign = -1
            elseif j > 0 then
                sign = 1
            end

            j = abs(j)

            local offsets = { 0, 1, 2, 3, 3, 2, 1, }

            offx = sign*offsets[(2*anim/3) % #offsets + 1]*FRACUNIT/2*j
            offy = -offsets[(2*anim/3) % #offsets + 1]*FRACUNIT
        end

        if font[c] then
            v.drawScaled(x*FRACUNIT + offx, y*FRACUNIT + offy, FRACUNIT, font[c], flags)
            x = x + font[c].width
        end
    end

    if v.interpolate then v.interpolate(false) end
end

local function getSnapFlags(splitnum)
    if splitscreen == 0 then
        return V_SNAPTOTOP
    end

    if splitscreen == 1 then
        if splitnum == 0 then
            return V_SNAPTOTOP
        end

        return 0
    end

    return (splitnum % 2 == 0) and V_SNAPTOLEFT or V_SNAPTORIGHT
end

local function getHudPos(splitnum)
    if splitscreen == 0 then
        return 130, 47
    end

    if splitscreen == 1 then
        if splitnum == 0 then
            return 130, 20
        end

        return 130, 120
    end

    local x, y

    if splitnum % 2 == 0 then
        x = 2
    else
        x = 320 - SMOLGAUGE_WIDTH - 2
    end

    if splitnum / 2 == 0 then
        y = 25
    else
        y = 200 - SMOLGAUGE_HEIGHT - 25
    end

    return x, y
end

local function getGaugeParams()
    if splitscreen < 2 then
        return GAUGE_WIDTH, GAUGE_HEIGHT, GAUGE_SPACE
    end

    return SMOLGAUGE_WIDTH, SMOLGAUGE_HEIGHT, SMOLGAUGE_SPACE
end

hud.add(function(v, p, c)
    if not font then
        cacheTrickFont(v)
    end

    local trans = v.localTransFlag()
    local gaugetrans = max(0, (trans>>V_ALPHASHIFT)-4)<<V_ALPHASHIFT

    if not p.mo then return end

    local rb = trickData(p.mo, true)

    if not rb or (rb.boosttimer <= 0 and not (p.deadtimer > 0 and rb and rb.trickbuffer) and not rb.trickmiss) then return end

    local snap = getSnapFlags(c.pnum-1)
    local x, y = getHudPos(c.pnum-1)
    local gaugew, gaugeh, gauges = getGaugeParams()

    -- Trick gauge part
    if p.deadtimer == 0 and rb.boosttimer > 0 then
        local level, fill = getGaugeLevel(rb)

        drawGaugeBackground(v, x, y, gaugew, gaugeh, trans|snap)

        local prevlevel = level - 1

        if prevlevel > 0 then
            drawGauge(v, x, y, gaugew, gaugeh, gauges, FRACUNIT, getGaugeColor(prevlevel), trans|snap)
        end

        local gaugecolor = getGaugeColor(level)

        if level == 1 then
            if fill < FRACUNIT/4 then
                gaugecolor = 130
            elseif fill < FRACUNIT/2 then
                gaugecolor = 100
            end
        end

        drawGauge(v, x, y, gaugew, gaugeh, gauges, fill, gaugecolor, gaugetrans|snap)
    end

    -- Trick string part
    local num = clamp(1, rb.trickcombo, 4)

    if p.deadtimer then num = 0 end
	if rb.trickmiss then num = 0 end

    if splitscreen < 2 then
        drawTrickString(v, x + GAUGE_WIDTH/2, y - 14, num, trans|snap)
    else
        v.drawString(x + ((c.pnum-1) % 2)*gaugew, y - 6, combocolor[num]..combotext[num], trans|snap, ((c.pnum-1) % 2 == 1) and "small-right" or "small")
    end
end)

-- Commands

COM_AddCommand("rb_trickbutton", function(p, choice)
	local names = {
		[BT_BRAKE] = "Brake",
		[BT_CUSTOM1] = "Custom 1",
		[BT_CUSTOM2] = "Custom 2",
		[BT_CUSTOM3] = "Custom 3",
	}

	if not choice then
		CONS_Printf(p, "Usage: \x81rb_trickbutton <button>\x80")
		CONS_Printf(p, "Possible buttons: brake (b), custom1 (c1), custom2 (c2), custom3 (c3)")
		
		local current = p.rb_trickbutton or BT_BRAKE
		
		CONS_Printf(p, "Current choice: \x83"..names[current])
		
		return
	end
	
	local buttons = {
		brake = BT_BRAKE,
		b = BT_BRAKE,
		custom1 = BT_CUSTOM1,
		c1 = BT_CUSTOM1,
		custom2 = BT_CUSTOM2,
		c2 = BT_CUSTOM2,
		custom3 = BT_CUSTOM3,
		c3 = BT_CUSTOM3,
	}
	
	local b = buttons[choice:lower()]
	
	if not b then
		CONS_Printf(p, "Unknown button, try rb_trickbutton without arguments for list of possible buttons")
		return
	end
	
	p.rb_trickbutton = b
	
	CONS_Printf(p, "Changed trick button to \x83"..names[b])
end)

COM_AddCommand("rb_holdaccel", function(p, choice)
	if not choice then
		CONS_Printf(p, "Usage: \x81rb_holdaccel <on/off>\x80")
		CONS_Printf(p, "Enabling that will hold accel for you when you have trick boost")
		CONS_Printf(p, "Holding accel when having trick boost is currently"..(p.rb_holdaccel and "\x83 enabled" or " \x85 disabled"))
		return
	end
	
	local options = {
		["1"] = true,
		["on"] = true,
		["yes"] = true,
		["true"] = true,
	}
	
	p.rb_holdaccel = options[choice]
	
	CONS_Printf(p, "Holding accel when having trick boost has been"..(p.rb_holdaccel and "\x83 enabled" or "\x85 disabled"))
end)

local function findTheme(choice)
	-- Try full match first
	for k, _ in pairs(themes) do
		if k == choice then
			return { k }, true
		end
	end
	
	local matches = {}

	for k, _ in pairs(themes) do
		if k:find(choice) then
			table.insert(matches, k)
		end
	end
	
	return matches, false
end

COM_AddCommand("rb_theme", function(p, choice)
	if not choice then
		CONS_Printf(p, "Usage: \x81rb_theme <name>\x80")
		CONS_Printf(p, "Current theme: \x83"..curtheme)
		
		local names = {}
		for k, _ in pairs(themes) do
			table.insert(names, k)
		end
		
		CONS_Printf(p, "Possible themes: \x83"..table.concat(names, ', '))
		
		return
	end
	
	local matches, full_match = findTheme(choice:lower())
	
	if #matches == 0 then
		CONS_Printf(p, "Theme not found, try rb_themes without arguments for list of themes")
		return
	end
	
	if not full_match and #matches > 1 then
		CONS_Printf(p, "Multiple matches found:")
		CONS_Printf(p, "\x83"..table.concat(matches, " "))
		CONS_Printf(p, "Try be more precise")
		return
	end
	
	curtheme = matches[1]
	
	CONS_Printf(p, "Set current theme to \x83"..curtheme)
end, COM_LOCAL)

-- Stuff for possible interop
--
-- Some functions are not exposed because i'm not sure about api consistency
-- (some functions take player_t instead of mobj_t, or also require rebasics table argument)
-- Feel free to add anything you need tho. But maybe also lemme know so i can refactor those functions to be more consistent with exposed ones

rawset(_G, "RB_TrickData", trickData)
rawset(_G, "RB_DoTrick", doTrick)
rawset(_G, "RB_ReleaseTrick", releaseTrick)
rawset(_G, "RB_TrickBoost", trickBoost)
rawset(_G, "RB_CanTrick", canTrick)
rawset(_G, "RB_AddTheme", function(name, t)
	-- We can replace themes, so we don't check if it doesn't exist
	-- Also convert name to lowercase for easier search
	themes[name:lower()] = t
end)